"""
This module contains the APIs for updating the STAC catalog for each app and each of it's file types.
These APIs need to be called by the app data curator for maintaining the STAC catalog.
"""

#Standard modules
import os
import sys
import json
import random
import itertools
from datetime import datetime, timezone, timedelta
import importlib

#Non-standard modules
try:
    #import rasterio
    import pystac
    from shapely.geometry import Polygon, mapping
except:
    print( sys.exc_info() )
    print( f"Module 'rasterio/pystac/shapely' import error in {__file__}" )


bbox_globals = [ [9.975688, 53.566570, 9.976192, 53.566863 ],
                 [9.976262, 53.566860, 9.976552, 53.566988 ],
                 [9.985612, 53.552917, 9.985794, 53.553038 ]
               ]

bbox_names=['Room23','Room34','Fuhlentwiete']

footprint_polygons = [ [ [ 9.975688, 53.566570 ], [ 9.975688, 53.566863 ], [ 9.976192, 53.566863 ], [ 9.976192, 53.566570 ], [ 9.975688, 53.566570 ] ],
                       [ [ 9.976262, 53.566860 ], [ 9.976552, 53.566860 ], [ 9.976552, 53.566988 ], [ 9.976262, 53.566988 ], [ 9.976262, 53.566860 ] ],
                       [ [ 9.985612, 53.552917 ], [ 9.985794, 53.552917 ], [ 9.985794, 53.553038 ], [ 9.985612, 53.553038 ], [ 9.985612, 53.552917 ] ]
                     ]

snacks = { 'Donauwelle': {'rezept': 'https://www.einfachbacken.de/rezepte/klassische-donauwelle-mit-pudding', 'baker':'Katharina' },
           'Pakora': {'rezept': 'https://elavegan.com/de/pakora-rezept/', 'baker':'Kameswar' },
           'Zimtschnecken': {'rezept': 'https://www.einfachbacken.de/rezepte/zimtschnecken-einfach-selbstgemacht', 'baker':'Andrea' },
           'Rhabarberkuchen': {'rezept': 'https://www.einfachbacken.de/rezepte/rhabarberkuchen-das-beste-rezept-mit-streusel', 'baker':'Heydebreck' }
         }

sourceMediaTypeMap =  {
    
   'netcdf':{
                '.nc' : pystac.MediaType.HDF5
    },
   
    'zarr':{
                '.zarr' : pystac.MediaType.ZARR
    },
   
    'image':{
                '.pdf' : pystac.MediaType.PDF,
                '.jpg' : pystac.MediaType.JPEG,
                '.png' : pystac.MediaType.PNG,
                '.tiff': pystac.MediaType.TIFF
    },
   
    'text':{
                '.csv' : pystac.MediaType.TEXT, # to check if any other driver exists.
                '.txt' : pystac.MediaType.TEXT
    },
   
    'html':{
                '.html' : pystac.MediaType.HTML
    }
}

tgifCollection = ['2024','2023','2022','2021','2020','2019','2018','2017']
#tgifCollection = ['2017']


def getCollectionDict(iyear):
    from itertools import zip_longest

    annual_dict={}
    files = [open(filename) for filename in [f"./input/{iyear}.dat",f"./input/{iyear}_b.dat",f"./input/{iyear}_c.dat"] ]
    #print(files)
    talkCount=1
    annual_dict[str(iyear)]={}

    for lines in zip_longest(*files, fillvalue=''):
        talkInfo=lines[0].replace('(pdf)','').replace('(ppt)','').strip('\n').split('\t')
        print(talkInfo)
        talkKey=f"talk{talkCount}"
        annual_dict[str(iyear)].update({talkKey:{'title':talkInfo[0], 'date':talkInfo[2], 'speakers':talkInfo[1], 'slides':lines[1].strip('\n'), 'thumbnail':lines[2].strip('\n') }  })
        talkCount+=1
    print(annual_dict)

    return annual_dict[iyear]



def getSpeakerLink(speakerName):
    import requests
    from bs4 import BeautifulSoup
    import re

    URL = 'https://www.dkrz.de/de/about/mitarbeiter/abt-datenmanagement'
    headers = {"User-Agent": 'Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:130.0) Gecko/20100101 Firefox/130.0' }
    page=requests.get(URL, headers=headers)
    soup=BeautifulSoup(page.content,'html.parser')

    for tag in soup.find_all('a'):
        #print(tag)
        if ('href' in tag.attrs and "mitarbeiter" in tag.attrs.get('href') ):
            #print(tag.attrs.get('href') )
            #speaker="angelika"
            if ('href' in tag.attrs and speakerName.casefold() in tag.attrs.get('href').casefold() ):
                #print(tag.attrs.get('href') )
                #print( speakerName )
                return tag.attrs.get('href')
    return None


def createTGIFSTACCatalog():
    """Create master STAC catalog file for the HPC centers.

    Create the top level master STAC catalog containing the links to the STAC catalogs for the HPC centers.

    Args:
            None.
    Returns: 
            None.
    """

    outputPath = "/home/kamesh/Dokumente/TGIF/STAC_for_TGIF"

    catalog_id='TGIF-stac'
    catalog_description='STAC catalog for the TGIF( Thank God It\'s Friday ) talks regularly held on Friday at 10:00 am in the Data Management department over the years.'
    catalog_title='TGIF talks Catalog'

    catalog = pystac.Catalog(id=f"{catalog_id}",title=f"{catalog_title}",description=f"{catalog_description}")
    

    for iyear in tgifCollection:

        spatial_extent = pystac.SpatialExtent(bboxes=[[-180, -90, 180, 90 ]])

        utc1 = datetime.strptime( f"{iyear}-01-01T00:00:00Z","%Y-%m-%dT%H:%M:%SZ" )
        utc2 = datetime.strptime( f"{iyear}-12-31T00:00:00Z","%Y-%m-%dT%H:%M:%SZ" )
        
        temporal_extent = pystac.TemporalExtent(intervals=[[utc1,utc2]]) # List of lists
        

        collection_extent = pystac.Extent( spatial=spatial_extent, temporal=temporal_extent )

        annualCollection = pystac.Collection( id = f'{iyear}', description = f"TGIF talks held in the year {iyear}.",
                        extent=collection_extent, license='CC-BY-SA-4.0' )

        updateAnnualCollections(iyear, annualCollection)

        catalog.add_child( annualCollection, f'{iyear}' )

    
    try:
        os.path.exists( outputPath )
    except:
        print( sys.exc_info() )
        return
        
    catalog.normalize_and_save( root_href = outputPath, 
                                catalog_type = pystac.CatalogType.SELF_CONTAINED )


def updateAnnualCollections( iyear, annualCollection ):

    iyear_dict = getCollectionDict( iyear )
    talkCount = len( iyear_dict.keys() )

    print(f"{iyear_dict} {talkCount}")

    

    for italk in range( 1, talkCount+1 ):

        italkInfo = iyear_dict[f'talk{italk}']
        print(italkInfo)

        italk_datetime = italkInfo['date']
        if italkInfo['date'] == '':
            datetime_utc = datetime.now( tz=timezone.utc )
        else:
            datetime_utc = datetime.strptime( italk_datetime, "%d.%m.%Y" ) + timedelta(hours=10)
        

        boxIndex = random.randrange(3)
        
        bbox_global = bbox_globals[boxIndex]
        
        footprint_polygon = Polygon( footprint_polygons[boxIndex] )
        
        
        footprint = mapping( footprint_polygon )

        
        #create metadata for item
        itemMetadata = {}
        itemMetadata.update({'venue':bbox_names[boxIndex]})
        
        
        """
        #  Fetch the 'metadata' values from the 'srcFile' by removing file suffix and splitting with '_'.
        fileName, fileExt = os.path.splitext( os.path.basename( srcFile ) )
        metadataValues = fileName.split( '_' )

        # NOTE: The metadata keys have fixed order as the file name parts seperated by '_'
        if len( metadataValues ) < len( expMetadataKeys ) :
            for i in range( len( metadataValues ), len( expMetadataKeys ) ):
                metadataValues.append( '-' )

        for key in expMetadataKeys:
            #if app == 'AQUA':
                itemMetadata[ key ] =  metadataValues[ expMetadataKeys.index( key ) ]
            # TODO for other apps
            #else:
            #    print('Metadata not available')

        """

        title = italkInfo['title']
        
        item = pystac.Item( id = f'{title}',
                            geometry = footprint,
                            bbox = bbox_global,
                            datetime = datetime_utc,
                            properties = itemMetadata )
        
        
        speakerNames = italkInfo['speakers'].split(',')
        itemMetadata.update({'speakers':speakerNames})
        
        #print(speakerNames)
        
        for speaker in speakerNames:
            speakerName = ''
            speaker = speaker.strip()
            #print(speaker)
            

            if len(speaker.split(' ')) > 1 :
                speakerName = (speaker.split(' '))[0]
                if speakerName == 'Daniel':
                    speakerName = (speaker.split(' '))[1]
                    if speakerName == 'Heydebreck':
                        speakerName = 'neumann'
            else:
                speakerName = speaker

            if speaker  == '':
                speakerName = 'Hannes'

            #print(speakerName)
            
            speakerURL = getSpeakerLink(speakerName.casefold())
            #print(f"speakerURL: {speakerURL}")

            if speakerURL is not None:
            
                item.add_asset( key = f'{speakerName}',
                    asset = pystac.Asset( href = speakerURL, roles=["Speaker"],media_type = sourceMediaTypeMap['html']['.html'] ) )
            


        
        slides = italkInfo['slides']
        item.add_asset( key = f'Slides',
            asset = pystac.Asset( href = slides, roles=["Slides"], media_type = sourceMediaTypeMap[ 'html' ]['.html'] ) )
        #itemMetadata.update({'slides':slides})


        snackName = list(snacks.keys())[boxIndex]
        baker = snacks[snackName]['baker']
        snackRezept = snacks[snackName]['rezept']
        itemMetadata.update({'snack':snackName})

        item.add_asset( key = f'{snackName}',
            asset = pystac.Asset( href = snackRezept, roles=["Recipe","Snack"], media_type = sourceMediaTypeMap[ 'html' ]['.html'] ) )
        

        
        #print(baker)
        bakerURL = getSpeakerLink(baker)
        #print(bakerURL)
        item.add_asset( key = f'{baker}',
            asset = pystac.Asset( href = bakerURL, roles=["Baker"], media_type = sourceMediaTypeMap[ 'html' ]['.html'] ) )
        itemMetadata.update({'baker':baker})

        item.add_asset( key = f'thumbnail',
            asset = pystac.Asset( href = italkInfo['thumbnail'], roles=["thumbnail"], media_type = sourceMediaTypeMap[ 'image' ]['.png'] ) )


        #title = italkInfo['title']
        #item.add_asset( key = f'Titel',
        #    asset = pystac.Asset( href = slides, media_type = sourceMediaTypeMap[ src ][ fileExt ] ) )
        
        
        annualCollection.add_item( item )
        

if __name__ == "__main__":
    createTGIFSTACCatalog()
